<?php
/*
Plugin Name: Truncate WP Debug Log
Plugin URI: wp-pizza.com
Description: Periodically (weekly) truncate wordpress debug.log to the last 10000 lines
Version: 1.0
Author: ollybach
Author URI: https://www.wp-pizza.com/wp-utilities


# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/
if ( ! defined( 'ABSPATH' ) ){ exit(); } // Exit if accessed directly



/*******************************************************************
*
*	[register activation/de-activation/uninstall hooks]
*
********************************************************************/
register_activation_hook( __FILE__ , array( 'TRUNCATE_WP_DEBUG', 'plugin_activate' ) );
register_deactivation_hook( __FILE__, array( 'TRUNCATE_WP_DEBUG', 'plugin_deactivate' ) );


/*******************************************************************
*
*	[set some file paths / strings]
*
********************************************************************/
define('TRUNCATE_WP_DEBUG_NAME', 'TRUNCATE WP DEBUG BY CRON');
define('TRUNCATE_WP_DEBUG_FILE', __FILE__);
define('TRUNCATE_WP_DEBUG_SCHEDULE', 'weekly');



/*########################################################################################################################
#
#
#	CLASS - TRUNCATE_WP_DEBUG
#
# 	@package TRUNCATE_WP_DEBUG
# 	@subpackage core
# 	@copyright Copyright (c) 2021, ollybach
# 	@since 1.0
#
########################################################################################################################*/
if (!class_exists( 'TRUNCATE_WP_DEBUG' )) {


	class TRUNCATE_WP_DEBUG{
		
		/*
			[debug_log_path - string] path to the debug log 
			
			
			[max_lines - integer]
			max lines to keep 
			10000 would still leave about a 2mb debug log		 
			which one would think will be enough to debug things
			going back a few days 			
			
			
			[max_bytes - integer]
			we will skip running of the cron if the debug.log
			is already larger than the bytes set 
			
			it really should be cleanedup first or the cronjob would consume 
			masses of memory by creating an array of potentially hunderds of millions of keys
			I've seen debug.logs of over 4GB .....			
			
			
		*/
		protected $parameters = array();
	
	
	/**************************************************************************************************
	#
	#
	#	[CONSTRUCT]
	#
	#
	**************************************************************************************************/
	    public function __construct(){
	
			/*
				make the parameters filerable
			*/
			$this -> parameters = apply_filters('truncate_wp_debug_parameters',array(
				'debug_log_path' => WP_CONTENT_DIR . '/debug.log',//standard WP debug logpath
				'max_lines' => 10000,//keep last 10000 log entries
				'max_bytes' => (10*1000000),//only process debug log files smaller than 10mb
			));
	
	    	/*
	    		set cron action hook
	    	*/
	    	add_action( 'truncate_wp_debug_cron' , array($this, 'truncate_wp_debug') );
	
	    }
	
	/**************************************************************************************************
	#
	#
	#	[METHODS]
	#
	#
	**************************************************************************************************/

		/*******************************************************
		*	[truncate the debug on cron run]
		*	@since 1.0
		*	@return void
		********************************************************/
	    public function truncate_wp_debug(){
	    	
			/*
				skip if it debug log doesnt exist or is empty
				also skip if the debug is already massivly large (let's say 10mb+) 
				really should be cleanedup first or the below will consume 
				masses of memory by creating an array of potentially millions of keys
				I've seen debug.logs of over 4GB .....
			*/
			if( !is_file($this->parameters['debug_log_path']) || 0 === filesize($this->parameters['debug_log_path']) || filesize($this->parameters['debug_log_path']) > $this->parameters['max_bytes']){
				# if we ever want to know why it did not run 
				#error_log(TRUNCATE_WP_DEBUG_NAME." - SKIPPED ");	
				return;	
			}
	
			/*
				split into array by lines and reverse 
			*/
			$lines = array_reverse( file( $this->parameters['debug_log_path'] ) );
			
			/*
				skip the rest if we have less lines than we want to keep anyway
				(checking for is empty for good measure again, just because we can)
			*/
			if(empty($lines) || count($lines) <= $this->parameters['max_lines']){
				# if we ever want to know why it did not run 
				#error_log(TRUNCATE_WP_DEBUG_NAME." - SKIPPED (Lines in log <= ".$this->parameters['max_lines']." ) ");	
				return;
			}
	
			/*
				chunk the array
			*/
			$lines = array_chunk($lines, $this->parameters['max_lines']);
	
			/*
				reset to only get first chunk
			*/
			$lines = reset($lines);
	
			/*
				reverse to original order
			*/
			$lines = array_reverse($lines);
	
			/*
				implode to text lines again
			*/
			$lines = implode('', $lines);
	
			/*
				write truncated to file
			*/
			@file_put_contents($this->parameters['debug_log_path'], $lines);
	
	
			/*
				register event in log
			*/
			error_log(TRUNCATE_WP_DEBUG_NAME." - RUN");	
	
		return;
	    }
		/*******************************************************
		*	[run on plugin activation
		*	- skipped if user has insufficient permissions]
		*	@since 1.0
		*	@return void
		********************************************************/
	    public static function plugin_activate(){
	        if ( ! current_user_can( 'activate_plugins' ) ){
	            return;
	        }
	        $referrer = isset( $_REQUEST['plugin'] ) ? 'activate-plugin_'.$_REQUEST['plugin'].'' : 'bulk-plugins';
	        check_admin_referer($referrer);
	
			/*
				add cron event
			*/
			if ( ! wp_next_scheduled( 'truncate_wp_debug_cron' ) ) {
			    /*
			    	dont run on activation of plugin
			    	but delay to 4AM (local time)tomorrow (+/- 1 hour depending on DST perhaps, but it's not really important)	
			    */
			    $date_time = (array)current_datetime();//just cast to array to make it "normally" accessible
				$date_time2str = strtotime($date_time['date']);
	
			    $next_schedule = strtotime('tomorrow', $date_time2str) + (4*60*60);//makeing sure next schedule is after utc time now or it will never run
			    //$next_schedule = time()+10;//temp for testing
				wp_schedule_event( $next_schedule, TRUNCATE_WP_DEBUG_SCHEDULE, 'truncate_wp_debug_cron',  array() );
			}
	
			/*
				register event in log
			*/
			error_log(TRUNCATE_WP_DEBUG_NAME." - ACTIVATED");
	    }
	
		/*******************************************************
		*	[run on plugin de-activation
		*	- skipped if user has insuficient permissions]
		*	@since 1.0
		*	@return void
		*
		********************************************************/
	    public static function plugin_deactivate(){
	        if ( ! current_user_can( 'activate_plugins' ) ){
	            return;
	        }
	        $referrer = isset( $_REQUEST['plugin'] ) ? 'deactivate-plugin_'.$_REQUEST['plugin'].'' : 'bulk-plugins';
	        check_admin_referer($referrer);        
	
	        /*
	        	clear cron event
	        */
	        wp_clear_scheduled_hook( 'truncate_wp_debug_cron', array() );
	
	        /*
	        	register event in log
	        */
	        error_log(TRUNCATE_WP_DEBUG_NAME." - DE-ACTIVATED");
	    }
		
	}
/**************************************************************************************************
#
#
#	[INI CLASS]
#
#
**************************************************************************************************/
$TRUNCATE_WP_DEBUG = new TRUNCATE_WP_DEBUG();
}
?>